@@ -8,6 +8,7 @@ module Database.Persist.QuasiSpec where
 import Prelude hiding (lines)
 
 import Control.Exception
+import Control.Monad
 import Data.List hiding (lines)
 import Data.List.NonEmpty (NonEmpty(..), (<|))
 import qualified Data.List.NonEmpty as NEL
@@ -84,8 +85,7 @@ spec = describe "Quasi" $ do
                     Nothing
         it "errors on invalid input" $ do
             evaluate (subject ["name", "int"])
-                `shouldThrow`
-                    errorCall "Invalid field type \"int\" PSFail ('i',\"nt\")"
+              `shouldErrorWithMessage` "Invalid field type \"int\" PSFail \"int\""
         it "works if it has a name and a type" $ do
             subject ["asdf", "Int"]
                 `shouldBe`
@@ -140,6 +140,15 @@ spec = describe "Quasi" $ do
                         ]
                     )
 
+        it "handles numbers" $
+            parseLine "  one (Finite 1)" `shouldBe`
+                Just
+                    ( Line 2
+                        [ Token "one"
+                        , Token "Finite 1"
+                        ]
+                    )
+
         it "handles quotes" $
             parseLine "  \"foo bar\"  \"baz\"" `shouldBe`
                 Just
@@ -151,8 +160,8 @@ spec = describe "Quasi" $ do
 
         it "should error if quotes are unterminated" $ do
             evaluate (parseLine "    \"foo bar")
-                `shouldThrow`
-                    errorCall "Unterminated quoted string starting with foo bar"
+                `shouldErrorWithMessage`
+                    "Unterminated quoted string starting with foo bar"
 
         it "handles quotes mid-token" $
             parseLine "  x=\"foo bar\"  \"baz\"" `shouldBe`
@@ -227,6 +236,13 @@ spec = describe "Quasi" $ do
                             [ DocComment "this is a comment"
                             ]
                         )
+            it "recognizes empty line" $ do
+                parseLine "-- |" `shouldBe`
+                    Just
+                        ( Line 0
+                            [ DocComment ""
+                            ]
+                        )
 
             it "works if comment is indented" $ do
                 parseLine "  -- | comment" `shouldBe`
@@ -357,8 +373,8 @@ User
 |]
             let [user] = parse lowerCaseSettings definitions
             evaluate (unboundEntityDef user)
-                `shouldThrow`
-                    errorCall "Unterminated parens string starting with Maybe Int"
+                `shouldErrorWithMessage`
+                    "Unterminated parens string starting with Maybe Int"
 
         it "errors on duplicate cascade update declarations" $ do
             let definitions = [st|
@@ -367,8 +383,8 @@ User
 |]
             let [user] = parse lowerCaseSettings definitions
             mapM (evaluate . unboundFieldCascade) (unboundEntityFields user)
-                `shouldThrow`
-                    errorCall "found more than one OnUpdate action, tokens: [\"OnUpdateCascade\",\"OnUpdateCascade\"]"
+                `shouldErrorWithMessage`
+                    "found more than one OnUpdate action, tokens: [\"OnUpdateCascade\",\"OnUpdateCascade\"]"
 
         it "errors on duplicate cascade delete declarations" $ do
             let definitions = [st|
@@ -377,8 +393,8 @@ User
 |]
             let [user] = parse lowerCaseSettings definitions
             mapM (evaluate . unboundFieldCascade) (unboundEntityFields user)
-                `shouldThrow`
-                    errorCall "found more than one OnDelete action, tokens: [\"OnDeleteCascade\",\"OnDeleteCascade\"]"
+                `shouldErrorWithMessage`
+                    "found more than one OnDelete action, tokens: [\"OnDeleteCascade\",\"OnDeleteCascade\"]"
 
         describe "custom Id column" $ do
             it "parses custom Id column" $ do
@@ -410,8 +426,8 @@ User
 |]
                 let [user] = parse lowerCaseSettings definitions
                     errMsg = [st|expected only one Id declaration per entity|]
-                evaluate (unboundEntityDef user) `shouldThrow`
-                    errorCall (T.unpack errMsg)
+                evaluate (unboundEntityDef user) `shouldErrorWithMessage`
+                    (T.unpack errMsg)
 
         describe "primary declaration" $ do
             it "parses Primary declaration" $ do
@@ -434,6 +450,18 @@ User
                     , FieldNameHS "name"
                     , FieldNameHS "age"
                     ]
+                entityUniques (unboundEntityDef user) `shouldBe`
+                    [ UniqueDef
+                        { uniqueHaskell =
+                            ConstraintNameHS "UserPrimaryKey"
+                        , uniqueDBName =
+                            ConstraintNameDB "primary_key"
+                        , uniqueFields =
+                            pure (FieldNameHS "ref", FieldNameDB "ref")
+                        , uniqueAttrs =
+                            []
+                        }
+                    ]
 
             it "errors on duplicate custom Primary declaration" $ do
                 let definitions = [st|
@@ -445,9 +473,9 @@ User
     Primary name
 |]
                 let [user] = parse lowerCaseSettings definitions
-                    errMsg = [st|expected only one Primary declaration per entity|]
-                evaluate (unboundEntityDef user) `shouldThrow`
-                    errorCall (T.unpack errMsg)
+                    errMsg = "expected only one Primary declaration per entity"
+                evaluate (unboundEntityDef user) `shouldErrorWithMessage`
+                    errMsg
 
             it "errors on conflicting Primary/Id declarations" $ do
                 let definitions = [st|
@@ -460,8 +488,8 @@ User
 |]
                 let [user] = parse lowerCaseSettings definitions
                     errMsg = [st|Specified both an ID field and a Primary field|]
-                evaluate (unboundEntityDef user) `shouldThrow`
-                    errorCall (T.unpack errMsg)
+                evaluate (unboundEntityDef user) `shouldErrorWithMessage`
+                    (T.unpack errMsg)
 
             it "triggers error on invalid declaration" $ do
                 let definitions = [st|
@@ -472,8 +500,8 @@ User
                 let [user] = parse lowerCaseSettings definitions
                 case unboundPrimarySpec user of
                     NaturalKey ucd -> do
-                        evaluate (head $ unboundCompositeCols ucd) `shouldThrow`
-                            errorCall "Unknown column in primary key constraint: \"ref\""
+                        evaluate (NEL.head $ unboundCompositeCols ucd) `shouldErrorWithMessage`
+                            "Unknown column in primary key constraint: \"ref\""
                     _ ->
                         error "Expected NaturalKey, failing"
 
@@ -493,8 +521,8 @@ User
                         [ "Unknown column in \"UniqueEmail\" constraint: \"emailSecond\""
                         , "possible fields: [\"name\",\"emailFirst\"]"
                         ]
-                evaluate (head (NEL.tail dbNames)) `shouldThrow`
-                    errorCall errMsg
+                evaluate (head (NEL.tail dbNames)) `shouldErrorWithMessage`
+                    errMsg
 
             it "triggers error if no valid constraint name provided" $ do
                 let definitions = [st|
@@ -503,8 +531,8 @@ User
     Unique some
 |]
                 let [user] = parse lowerCaseSettings definitions
-                evaluate (unboundPrimarySpec user) `shouldThrow`
-                    errorCall "invalid unique constraint on table[\"User\"] expecting an uppercase constraint name xs=[\"some\"]"
+                evaluate (unboundPrimarySpec user) `shouldErrorWithMessage`
+                    "invalid unique constraint on table[\"User\"] expecting an uppercase constraint name xs=[\"some\"]"
 
         describe "foreign keys" $ do
             let validDefinitions = [st|
@@ -552,8 +580,8 @@ Notification
 |]
                 let [_user, notification] = parse (setPsUseSnakeCaseForiegnKeys lowerCaseSettings) definitions
                 mapM (evaluate . unboundForeignFields) (unboundForeignDefs notification)
-                    `shouldThrow`
-                        errorCall "invalid foreign key constraint on table[\"Notification\"] expecting a lower case constraint name or a cascading action xs=[]"
+                    `shouldErrorWithMessage`
+                        "invalid foreign key constraint on table[\"Notification\"] expecting a lower case constraint name or a cascading action xs=[]"
 
             it "should error when foreign fields not provided" $ do
                 let definitions = [st|
@@ -572,8 +600,8 @@ Notification
 |]
                 let [_user, notification] = parse (setPsUseSnakeCaseForiegnKeys lowerCaseSettings) definitions
                 mapM (evaluate . unboundForeignFields) (unboundForeignDefs notification)
-                    `shouldThrow`
-                        errorCall "No fields on foreign reference."
+                    `shouldErrorWithMessage`
+                        "No fields on foreign reference."
 
             it "should error when number of parent and foreign fields differ" $ do
                 let definitions = [st|
@@ -592,8 +620,8 @@ Notification
 |]
                 let [_user, notification] = parse (setPsUseSnakeCaseForiegnKeys lowerCaseSettings) definitions
                 mapM (evaluate . unboundForeignFields) (unboundForeignDefs notification)
-                    `shouldThrow`
-                        errorCall "invalid foreign key constraint on table[\"Notification\"] Found 2 foreign fields but 1 parent fields"
+                    `shouldErrorWithMessage`
+                        "invalid foreign key constraint on table[\"Notification\"] Found 2 foreign fields but 1 parent fields"
 
             it "should throw error when there is more than one delete cascade on the declaration" $ do
                 let definitions = [st|
@@ -612,8 +640,8 @@ Notification
 |]
                 let [_user, notification] = parse (setPsUseSnakeCaseForiegnKeys lowerCaseSettings) definitions
                 mapM (evaluate . unboundForeignFields) (unboundForeignDefs notification)
-                    `shouldThrow`
-                        errorCall "invalid foreign key constraint on table[\"Notification\"] found more than one OnDelete actions"
+                    `shouldErrorWithMessage`
+                        "invalid foreign key constraint on table[\"Notification\"] found more than one OnDelete actions"
 
             it "should throw error when there is more than one update cascade on the declaration" $ do
                 let definitions = [st|
@@ -632,8 +660,8 @@ Notification
 |]
                 let [_user, notification] = parse (setPsUseSnakeCaseForiegnKeys lowerCaseSettings) definitions
                 mapM (evaluate . unboundForeignFields) (unboundForeignDefs notification)
-                    `shouldThrow`
-                        errorCall  "invalid foreign key constraint on table[\"Notification\"] found more than one OnUpdate actions"
+                    `shouldErrorWithMessage`
+                        "invalid foreign key constraint on table[\"Notification\"] found more than one OnUpdate actions"
 
             it "should allow you to enable snake cased foriegn keys via a preset configuration function" $ do
                 let [_user, notification] =
@@ -666,6 +694,22 @@ CustomerTransfer
                     , (FieldNameHS "uuid", FTTypeCon Nothing "TransferUuid")
                     ]
 
+        describe "type literals" $ do
+            it "should be able to parse type literals" $ do
+                let simplifyField field =
+                        (unboundFieldNameHS field, unboundFieldType field)
+                let tickedDefinition = [st|
+WithFinite
+    one    (Finite 1)
+    twenty (Labelled "twenty")
+|]
+                let [withFinite] = parse lowerCaseSettings tickedDefinition
+
+                (simplifyField <$> unboundEntityFields withFinite) `shouldBe`
+                    [ (FieldNameHS "one", FTApp (FTTypeCon Nothing "Finite") (FTLit (IntTypeLit 1)))
+                    , (FieldNameHS "twenty", FTApp (FTTypeCon Nothing "Labelled") (FTLit (TextTypeLit "twenty")))
+                    ]
+
     describe "parseFieldType" $ do
         it "simple types" $
             parseFieldType "FooBar" `shouldBe` Right (FTTypeCon Nothing "FooBar")
@@ -692,6 +736,12 @@ CustomerTransfer
                 baz = FTTypeCon Nothing "Baz"
             parseFieldType "Foo [Bar] Baz" `shouldBe` Right (
                 foo `FTApp` bars `FTApp` baz)
+        it "numeric type literals" $ do
+            let expected = FTApp (FTTypeCon Nothing "Finite") (FTLit (IntTypeLit 1))
+            parseFieldType "Finite 1" `shouldBe` Right expected
+        it "string type literals" $ do
+            let expected = FTApp (FTTypeCon Nothing "Labelled") (FTLit (TextTypeLit "twenty"))
+            parseFieldType "Labelled \"twenty\"" `shouldBe` Right expected
         it "nested list / parens (list inside parens)" $ do
             let maybeCon = FTTypeCon Nothing "Maybe"
                 int = FTTypeCon Nothing "Int"
@@ -703,7 +753,7 @@ CustomerTransfer
             parseFieldType "[Maybe (Maybe Int)]" `shouldBe` Right
                 (FTList (maybeCon `FTApp` (maybeCon `FTApp` int)))
         it "fails on lowercase starts" $ do
-            parseFieldType "nothanks" `shouldBe` Left "PSFail ('n',\"othanks\")"
+            parseFieldType "nothanks" `shouldBe` Left "PSFail \"nothanks\""
 
     describe "#1175 empty entity" $ do
         let subject =
@@ -1175,3 +1225,12 @@ Baz
 arbitraryWhiteSpaceChar :: Gen Char
 arbitraryWhiteSpaceChar =
   oneof $ pure <$> [' ', '\t', '\n', '\r']
+
+shouldErrorWithMessage :: IO a -> String -> Expectation
+shouldErrorWithMessage action expectedMsg = do
+    res <- try action
+    case res of
+        Left (ErrorCall msg) ->
+            msg `shouldBe` expectedMsg
+        _ ->
+            expectationFailure "Expected `error` to have been called"
